# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Decoder error
"""

NONE: int = 0
"""
No error. The last decoded instruction is a valid instruction
"""
INVALID_INSTRUCTION: int = 1
"""
It's an invalid instruction or an invalid encoding of an existing instruction (eg. some reserved bit is set/cleared)
"""
NO_MORE_BYTES: int = 2
"""
There's not enough bytes left to decode the instruction
"""
