# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
The code size (16/32/64) that was used when an instruction was decoded
"""

UNKNOWN: int = 0
"""
Unknown size
"""
CODE16: int = 1
"""
16-bit code
"""
CODE32: int = 2
"""
32-bit code
"""
CODE64: int = 3
"""
64-bit code
"""
