# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JNP`` / ``JPO``)
"""

NP: int = 0
"""
``JNP``, ``CMOVNP``, ``SETNP``
"""
PO: int = 1
"""
``JPO``, ``CMOVPO``, ``SETPO``
"""
