# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JBE`` / ``JNA``)
"""

BE: int = 0
"""
``JBE``, ``CMOVBE``, ``SETBE``
"""
NA: int = 1
"""
``JNA``, ``CMOVNA``, ``SETNA``
"""
