# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JA`` / ``JNBE``)
"""

A: int = 0
"""
``JA``, ``CMOVA``, ``SETA``
"""
NBE: int = 1
"""
``JNBE``, ``CMOVNBE``, ``SETNBE``
"""
