# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JE`` / ``JZ``)
"""

E: int = 0
"""
``JE``, ``CMOVE``, ``SETE``, ``LOOPE``, ``REPE``
"""
Z: int = 1
"""
``JZ``, ``CMOVZ``, ``SETZ``, ``LOOPZ``, ``REPZ``
"""
