# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JLE`` / ``JNG``)
"""

LE: int = 0
"""
``JLE``, ``CMOVLE``, ``SETLE``
"""
NG: int = 1
"""
``JNG``, ``CMOVNG``, ``SETNG``
"""
