# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JGE`` / ``JNL``)
"""

GE: int = 0
"""
``JGE``, ``CMOVGE``, ``SETGE``
"""
NL: int = 1
"""
``JNL``, ``CMOVNL``, ``SETNL``
"""
