# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JL`` / ``JNGE``)
"""

L: int = 0
"""
``JL``, ``CMOVL``, ``SETL``
"""
NGE: int = 1
"""
``JNGE``, ``CMOVNGE``, ``SETNGE``
"""
