# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Memory size options used by the formatters
"""

DEFAULT: int = 0
"""
Show memory size if the assembler requires it, else don't show anything
"""
ALWAYS: int = 1
"""
Always show the memory size, even if the assembler doesn't need it
"""
MINIMAL: int = 2
"""
Show memory size if a human can't figure out the size of the operand
"""
NEVER: int = 3
"""
Never show memory size
"""
