# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Instruction condition code (used by ``Jcc``, ``SETcc``, ``CMOVcc``, ``LOOPcc``)
"""

NONE: int = 0
"""
The instruction doesn't have a condition code
"""
O: int = 1
"""
Overflow (``OF=1``)
"""
NO: int = 2
"""
Not overflow (``OF=0``)
"""
B: int = 3
"""
Below (unsigned) (``CF=1``)
"""
AE: int = 4
"""
Above or equal (unsigned) (``CF=0``)
"""
E: int = 5
"""
Equal / zero (``ZF=1``)
"""
NE: int = 6
"""
Not equal / zero (``ZF=0``)
"""
BE: int = 7
"""
Below or equal (unsigned) (``CF=1 or ZF=1``)
"""
A: int = 8
"""
Above (unsigned) (``CF=0 and ZF=0``)
"""
S: int = 9
"""
Signed (``SF=1``)
"""
NS: int = 10
"""
Not signed (``SF=0``)
"""
P: int = 11
"""
Parity (``PF=1``)
"""
NP: int = 12
"""
Not parity (``PF=0``)
"""
L: int = 13
"""
Less (signed) (``SF!=OF``)
"""
GE: int = 14
"""
Greater than or equal (signed) (``SF=OF``)
"""
LE: int = 15
"""
Less than or equal (signed) (``ZF=1 or SF!=OF``)
"""
G: int = 16
"""
Greater (signed) (``ZF=0 and SF=OF``)
"""
