# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Formatter syntax (GNU Assembler, Intel XED, masm, nasm)
"""

GAS: int = 0
"""
GNU Assembler (AT&T)
"""
INTEL: int = 1
"""
Intel XED
"""
MASM: int = 2
"""
masm
"""
NASM: int = 3
"""
nasm
"""
