# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JB`` / ``JC`` / ``JNAE``)
"""

B: int = 0
"""
``JB``, ``CMOVB``, ``SETB``
"""
C: int = 1
"""
``JC``, ``CMOVC``, ``SETC``
"""
NAE: int = 2
"""
``JNAE``, ``CMOVNAE``, ``SETNAE``
"""
