# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JNE`` / ``JNZ``)
"""

NE: int = 0
"""
``JNE``, ``CMOVNE``, ``SETNE``, ``LOOPNE``, ``REPNE``
"""
NZ: int = 1
"""
``JNZ``, ``CMOVNZ``, ``SETNZ``, ``LOOPNZ``, ``REPNZ``
"""
