# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Rounding control
"""

NONE: int = 0
"""
No rounding mode
"""
ROUND_TO_NEAREST: int = 1
"""
Round to nearest (even)
"""
ROUND_DOWN: int = 2
"""
Round down (toward -inf)
"""
ROUND_UP: int = 3
"""
Round up (toward +inf)
"""
ROUND_TOWARD_ZERO: int = 4
"""
Round toward zero (truncate)
"""
