# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JP`` / ``JPE``)
"""

P: int = 0
"""
``JP``, ``CMOVP``, ``SETP``
"""
PE: int = 1
"""
``JPE``, ``CMOVPE``, ``SETPE``
"""
