# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Opcode table
"""

NORMAL: int = 0
"""
Legacy encoding table
"""
T0F: int = 1
"""
``0Fxx`` table (legacy, VEX, EVEX)
"""
T0F38: int = 2
"""
``0F38xx`` table (legacy, VEX, EVEX)
"""
T0F3A: int = 3
"""
``0F3Axx`` table (legacy, VEX, EVEX)
"""
XOP8: int = 4
"""
``XOP8`` table (XOP)
"""
XOP9: int = 5
"""
``XOP9`` table (XOP)
"""
XOPA: int = 6
"""
``XOPA`` table (XOP)
"""
