# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
``REP``/``REPE``/``REPNE`` prefix
"""

NONE: int = 0
"""
No ``REP``/``REPE``/``REPNE`` prefix
"""
REPE: int = 1
"""
``REP``/``REPE`` prefix
"""
REPNE: int = 2
"""
``REPNE`` prefix
"""
