# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Format mnemonic options
"""

NONE: int = 0x0000_0000
"""
No option is set
"""
NO_PREFIXES: int = 0x0000_0001
"""
Don't add any prefixes
"""
NO_MNEMONIC: int = 0x0000_0002
"""
Don't add the mnemonic
"""
