# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Instruction encoding
"""

LEGACY: int = 0
"""
Legacy encoding
"""
VEX: int = 1
"""
VEX encoding
"""
EVEX: int = 2
"""
EVEX encoding
"""
XOP: int = 3
"""
XOP encoding
"""
D3NOW: int = 4
"""
3DNow! encoding
"""
