# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JG`` / ``JNLE``)
"""

G: int = 0
"""
``JG``, ``CMOVG``, ``SETG``
"""
NLE: int = 1
"""
``JNLE``, ``CMOVNLE``, ``SETNLE``
"""
